@@ -60,24 +60,20 @@ thread_mbox (char *str)
 #endif
 
 /* leave this before XSUB.h, to avoid readdir() being redefined */
+
 static void
-perl_auto_load (void)
+perl_auto_load_from_path (const char *path)
 {
 	DIR *dir;
 	struct dirent *ent;
-	const char *xdir;
 
-	/* get the dir in local filesystem encoding (what opendir() expects!) */
-	xdir = xchat_get_info (ph, "xchatdirfs");
-	if (!xdir)						  /* xchatdirfs is new for 2.0.9, will fail on older */
-		xdir = xchat_get_info (ph, "xchatdir");
-	dir = opendir (xdir);
+	dir = opendir (path);
 	if (dir) {
 		while ((ent = readdir (dir))) {
 			int len = strlen (ent->d_name);
 			if (len > 3 && strcasecmp (".pl", ent->d_name + len - 3) == 0) {
-				char *file = malloc (len + strlen (xdir) + 2);
-				sprintf (file, "%s/%s", xdir, ent->d_name);
+				char *file = malloc (len + strlen (path) + 2);
+				sprintf (file, "%s/%s", path, ent->d_name);
 				perl_load_file (file);
 				free (file);
 			}
@@ -86,6 +82,26 @@ perl_auto_load (void)
 	}
 }
 
+static void
+perl_auto_load (void)
+{
+	const char *xdir;
+
+	/* get the dir in local filesystem encoding (what opendir() expects!) */
+	xdir = xchat_get_info (ph, "xchatdirfs");
+	if (!xdir)						  /* xchatdirfs is new for 2.0.9, will fail on older */
+		xdir = xchat_get_info (ph, "xchatdir");
+
+	/* autoload from ~/.xchat2/ or ${APPDATA}\X-Chat 2\ on win32 */
+	perl_auto_load_from_path (xdir);
+
+#ifdef WIN32
+	/* autoload from  C:\program files\xchat\plugins\ */
+	perl_auto_load_from_path (XCHATLIBDIR"/plugins");
+#endif
+	
+}
+
 #include <EXTERN.h>
 #define WIN32IOP_H
 #include <perl.h>
@@ -376,8 +392,10 @@ print_cb (char *word[], void *userdata)
 	}
 
 	for (count = 1; count <= last_index; count++) {
-		if (word[count] == NULL || word[count][0] == 0) {
+		if (word[count] == NULL) {
 			av_push (wd, &PL_sv_undef);
+		} else if (word[count][0] == 0) {
+			av_push (wd, newSVpvn ("",0));	
 		} else {
 			av_push (wd, newSVpv (word[count], 0));
 		}
@@ -1068,7 +1086,7 @@ perl_init (void)
 	   the xs_init function. (TheHobbit) */
 	int warn;
 	char *perl_args[] = { "", "-e", "0", "-w" };
-	const char perl_definitions[] = {
+	static const char perl_definitions[] = {
 		/* Redefine the $SIG{__WARN__} handler to have XChat
 		   printing warnings in the main window. (TheHobbit) */
 #include "xchat.pm.h"
@@ -1158,7 +1176,7 @@ perl_command_unloadall (char *word[], char *word_eol[], void *userdata)
 		return XCHAT_EAT_XCHAT;
 	}
 
-	return XCHAT_EAT_NONE;
+	return XCHAT_EAT_XCHAT;
 }
 
 static int
@@ -1170,7 +1188,7 @@ perl_command_reloadall (char *word[], char *word_eol[], void *userdata)
 
 		return XCHAT_EAT_XCHAT;
 	}
-	return XCHAT_EAT_NONE;
+	return XCHAT_EAT_XCHAT;
 }
 
 static int
@@ -1232,15 +1250,13 @@ perl_command_reload (char *word[], char *word_eol[], void *userdata)
 
 		len = strlen (file);
 
-
-		len = strlen (word_eol[2]);
 		if (len > 3 && strcasecmp (".pl", file + len - 3) == 0) {
 			execute_perl (sv_2mortal (newSVpv ("Xchat::Embed::reload", 0)),
 							  file);
 			return XCHAT_EAT_XCHAT;
 		}
 	}
-	return XCHAT_EAT_NONE;
+	return XCHAT_EAT_XCHAT;
 }
 
 void
@@ -1273,7 +1289,9 @@ xchat_plugin_init (xchat_plugin * plugin_handle, char **plugin_name,
 	}
 	initialized = 1;
 
-	xchat_plugin_get_info (plugin_name, plugin_desc, plugin_version, NULL);
+	*plugin_name = "Perl";
+	*plugin_desc = "Perl scripting interface";
+	*plugin_version = VERSION;
 
 	xchat_hook_command (ph, "load", XCHAT_PRI_NORM, perl_command_load, 0, 0);
 	xchat_hook_command (ph, "unload", XCHAT_PRI_NORM, perl_command_unload, 0,
