@@ -54,6 +54,11 @@
 #include "url.h"
 #include "xchatc.h"
 
+#ifdef USE_DCC64
+#define BIG_STR_TO_INT(x) strtoull(x,NULL,10)
+#else
+#define BIG_STR_TO_INT(x) strtoul(x,NULL,10)
+#endif
 
 static char *dcctypes[] = { "SEND", "RECV", "CHAT", "CHAT" };
 
@@ -77,7 +82,15 @@ static gboolean dcc_read_ack (GIOChannel *source, GIOCondition condition, struct
 
 static int new_id()
 {
-	static int id = 1;
+	static int id = 0;
+	if (id == 0)
+	{
+		/* start the first ID at a random number for pseudo security */
+		srand (time (0));
+		/* 1 - 255 */
+		id = 1 + (int) ((float)255 * rand()/(RAND_MAX+1.0));
+		/* ignore overflows, since it can go to 2 billion */
+	}
 	return id++;
 }
 
@@ -113,7 +126,7 @@ dcc_calc_cps (struct DCC *dcc)
 	double timediff, startdiff;
 	int glob_throttle_bit, wasthrottled;
 	int *cpssum, glob_limit;
-	unsigned int pos, posdiff;
+	DCC_SIZE pos, posdiff;
 
 	g_get_current_time (&now);
 
@@ -455,6 +468,8 @@ dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 	gsize utf_len;
 
 	len = strlen (line);
+	if (dcc->serv->using_cp1255)
+		len++;	/* include the NUL terminator */
 
 	if (dcc->serv->encoding == NULL)     /* system */
 		utf = g_locale_to_utf8 (line, len, NULL, &utf_len, NULL);
@@ -467,6 +482,9 @@ dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 		len = utf_len;
 	}
 
+	if (dcc->serv->using_cp1255 && len > 0)
+		len--;
+
 	/* we really need valid UTF-8 now */
 	conv = text_validate (&line, &len);
 
@@ -506,14 +524,14 @@ dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 		return 0;
 	}
 
-	url_check (line);
+	url_check_line (line, len);
 
 	if (line[0] == 1 && !strncasecmp (line + 1, "ACTION", 6))
 	{
 		po = strchr (line + 8, '\001');
 		if (po)
 			po[0] = 0;
-		inbound_action (sess, dcc->serv->nick, dcc->nick, line + 8, FALSE);
+		inbound_action (sess, dcc->serv->nick, dcc->nick, line + 8, FALSE, FALSE);
 	} else
 	{
 		inbound_privmsg (dcc->serv, dcc->nick, "", line, FALSE);
@@ -550,7 +568,7 @@ dcc_read_chat (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 		{
 			if (len < 0)
 			{
-				if (would_block_again ())
+				if (would_block ())
 					return TRUE;
 			}
 			sprintf (tbuf, "%d", dcc->port);
@@ -616,7 +634,7 @@ static void
 dcc_send_ack (struct DCC *dcc)
 {
 	/* send in 32-bit big endian */
-	guint32 pos = htonl (dcc->pos);
+	guint32 pos = htonl (dcc->pos & 0xffffffff);
 	send (dcc->sok, (char *) &pos, 4, 0);
 }
 
@@ -694,7 +712,7 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 		{
 			if (n < 0)
 			{
-				if (would_block_again ())
+				if (would_block ())
 				{
 					if (need_ack)
 						dcc_send_ack (dcc);
@@ -738,6 +756,20 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	}
 }
 
+static void
+dcc_open_query (session *sess, char *nick)
+{
+	char *cmd;
+
+	if (prefs.autodialog)
+	{
+		cmd = malloc (8 + strlen (nick));
+		sprintf (cmd, "query %s", nick);
+		handle_command (sess, cmd, FALSE);
+		free (cmd);
+	}
+}
+
 static gboolean
 dcc_connect_finished (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 {
@@ -813,7 +845,9 @@ dcc_connect_finished (GIOChannel *source, GIOCondition condition, struct DCC *dc
 		EMIT_SIGNAL (XP_TE_DCCCONSEND, dcc->serv->front_session,
 						 dcc->nick, host, dcc->file, NULL, 0);
 		break;
-	case TYPE_CHATRECV:
+	case TYPE_CHATSEND:	/* pchat */
+		dcc_open_query (dcc->serv->server_session, dcc->nick);
+	case TYPE_CHATRECV:	/* normal chat */
 		dcc->iotag = fe_input_add (dcc->sok, FIA_READ|FIA_EX, dcc_read_chat, dcc);
 		dcc->dccchat = malloc (sizeof (struct dcc_chat));
 		dcc->dccchat->pos = 0;
@@ -852,11 +886,11 @@ dcc_connect (struct DCC *dcc)
 		/* possible problems with filenames containing spaces? */
 		if (dcc->type == TYPE_RECV)
 			snprintf (tbuf, sizeof (tbuf), strchr (dcc->file, ' ') ?
-					"DCC SEND \"%s\" %lu %d %u %d" :
-					"DCC SEND %s %lu %d %u %d", dcc->file,
+					"DCC SEND \"%s\" %u %d %"DCC_SFMT" %d" :
+					"DCC SEND %s %u %d %"DCC_SFMT" %d", dcc->file,
 					dcc->addr, dcc->port, dcc->size, dcc->pasvid);
 		else
-			snprintf (tbuf, sizeof (tbuf), "DCC CHAT chat %lu %d %d",
+			snprintf (tbuf, sizeof (tbuf), "DCC CHAT chat %u %d %d",
 				dcc->addr, dcc->port, dcc->pasvid);
 		dcc->serv->p_ctcp (dcc->serv, dcc->nick, tbuf);
 	}
@@ -957,7 +991,7 @@ dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	{
 		if (len < 0)
 		{
-			if (would_block_again ())
+			if (would_block ())
 				return TRUE;
 		}
 		EMIT_SIGNAL (XP_TE_DCCSENDFAIL, dcc->serv->front_session,
@@ -971,26 +1005,38 @@ dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	recv (sok, (char *) &ack, 4, 0);
 	dcc->ack = ntohl (ack);
 
-	/* fix for BitchX */
-	if (dcc->ack < dcc->resumable)
-		dcc->ackoffset = TRUE;
-	if (dcc->ackoffset)
-		dcc->ack += dcc->resumable;
+	/* this could mess up when xfering >32bit files */
+	if (dcc->size <= 0xffffffff)
+	{
+		/* fix for BitchX */
+		if (dcc->ack < dcc->resumable)
+			dcc->ackoffset = TRUE;
+		if (dcc->ackoffset)
+			dcc->ack += dcc->resumable;
+	}
 
 	/* DCC complete check */
-	if (dcc->pos >= dcc->size && dcc->ack >= dcc->size)
+	if (dcc->pos >= dcc->size && dcc->ack >= (dcc->size & 0xffffffff))
 	{
+		dcc->ack = dcc->size;	/* force 100% ack for >4 GB */
 		dcc_calc_average_cps (dcc);
 		dcc_close (dcc, STAT_DONE, FALSE);
 		sprintf (buf, "%d", dcc->cps);
 		EMIT_SIGNAL (XP_TE_DCCSENDCOMP, dcc->serv->front_session,
 						 file_part (dcc->file), dcc->nick, buf, NULL, 0);
 	}
-	else if ((!dcc->fastsend) && (dcc->ack >= dcc->pos))
+	else if ((!dcc->fastsend) && (dcc->ack >= (dcc->pos & 0xffffffff)))
 	{
 		dcc_send_data (NULL, 0, (gpointer)dcc);
 	}
 
+#ifdef USE_DCC64
+	/* take the top 32 of "bytes send" and bottom 32 of "ack" */
+	dcc->ack = (dcc->pos & G_GINT64_CONSTANT (0xffffffff00000000)) |
+					(dcc->ack & 0xffffffff);
+	/* dcc->ack is only used for CPS and PERCENTAGE calcs from now on... */
+#endif
+
 	return TRUE;
 }
 
@@ -1038,13 +1084,7 @@ dcc_accept (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 		break;
 
 	case TYPE_CHATSEND:
-		if (prefs.autodialog)
-		{
-			char *cmd = malloc (8 + strlen (dcc->nick));
-			sprintf (cmd, "query %s", dcc->nick);
-			handle_command (dcc->serv->server_session, cmd, FALSE);
-			free (cmd);
-		}
+		dcc_open_query (dcc->serv->server_session, dcc->nick);
 		dcc->iotag = fe_input_add (dcc->sok, FIA_READ|FIA_EX, dcc_read_chat, dcc);
 		dcc->dccchat = malloc (sizeof (struct dcc_chat));
 		dcc->dccchat->pos = 0;
@@ -1058,12 +1098,35 @@ dcc_accept (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	return TRUE;
 }
 
+guint32
+dcc_get_my_address (void)	/* the address we'll tell the other person */
+{
+	struct hostent *dns_query;
+	guint32 addr = 0;
+
+	if (prefs.ip_from_server && prefs.dcc_ip)
+		addr = prefs.dcc_ip;
+	else if (prefs.dcc_ip_str[0])
+	{
+	   dns_query = gethostbyname ((const char *) prefs.dcc_ip_str);
+
+	   if (dns_query != NULL &&
+	       dns_query->h_length == 4 &&
+	       dns_query->h_addr_list[0] != NULL)
+		{
+			/*we're offered at least one IPv4 address: we take the first*/
+			addr = *((guint32*) dns_query->h_addr_list[0]);
+		}
+	}
+
+	return addr;
+}
+
 static int
 dcc_listen_init (struct DCC *dcc, session *sess)
 {
-	unsigned long my_addr;
+	guint32 my_addr;
 	struct sockaddr_in SAddr;
-	struct hostent *dns_query;
 	int i, bindretval = -1;
 	socklen_t len;
 
@@ -1128,24 +1191,7 @@ dcc_listen_init (struct DCC *dcc, session *sess)
 	/*if we have a dcc_ip, we use that, so the remote client can connect*/
 	/*else we try to take an address from dcc_ip_str*/
 	/*if something goes wrong we tell the client to connect to our LAN ip*/
-
-	dcc->addr = 0;
-
-	if (prefs.ip_from_server != 0 && prefs.dcc_ip != 0)
-		dcc->addr = prefs.dcc_ip;
-	else if (prefs.dcc_ip_str[0])
-	{
-	   dns_query = gethostbyname ((const char *) prefs.dcc_ip_str);
-
-	   if (dns_query != NULL &&
-	       dns_query->h_length == 4 &&
-	       dns_query->h_addr_list[0] != NULL)
-		  {
-		    /*we're offered at least one IPv4 address: we take the first*/
-
-		    dcc->addr = (unsigned long) *((guint32*) dns_query->h_addr_list[0]);
-		  }
-	}
+	dcc->addr = dcc_get_my_address ();
 
 	/*if nothing else worked we use the address we bound to*/
 	if (dcc->addr == 0)
@@ -1225,11 +1271,16 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 
 	if (stat (file_fs, &st) != -1)
 	{
+
+#ifndef USE_DCC64
 		if (sizeof (st.st_size) > 4 && st.st_size > 4294967295U)
 		{
 			PrintText (sess, "Cannot send files larger than 4 GB.\n");
-			goto noaxs;
+			g_free (file_fs);
+			dcc_close (dcc, 0, TRUE);
+			return;
 		}
+#endif
 
 		if (*file_part (file_fs) && !S_ISDIR (st.st_mode))
 		{
@@ -1271,15 +1322,15 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 						{
 							dcc->pasvid = new_id();
 							snprintf (outbuf, sizeof (outbuf), (havespaces) ?
-									"DCC SEND \"%s\" %lu %d %u %d" :
-									"DCC SEND %s %lu %d %u %d",
-									file_part (dcc->file), 199ul,
+									"DCC SEND \"%s\" 199 %d %"DCC_SFMT" %d" :
+									"DCC SEND %s 199 %d %"DCC_SFMT" %d",
+									file_part (dcc->file),
 									0, dcc->size, dcc->pasvid);
 						} else
 						{
 							snprintf (outbuf, sizeof (outbuf), (havespaces) ?
-									"DCC SEND \"%s\" %lu %d %u" :
-									"DCC SEND %s %lu %d %u",
+									"DCC SEND \"%s\" %u %d %"DCC_SFMT :
+									"DCC SEND %s %u %d %"DCC_SFMT,
 									file_part (dcc->file), dcc->addr,
 									dcc->port, dcc->size);
 						}
@@ -1297,7 +1348,6 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 		}
 	}
 	PrintTextf (sess, _("Cannot access %s\n"), dcc->file);
-noaxs:
 	g_free (file_fs);
 	dcc_close (dcc, 0, TRUE);
 }
@@ -1311,14 +1361,13 @@ find_dcc_from_id (int id, int type)
 	{
 		dcc = (struct DCC *) list->data;
 		if (dcc->pasvid == id &&
-		dcc->dccstat == STAT_QUEUED && dcc->type == type)
+			 dcc->dccstat == STAT_QUEUED && dcc->type == type)
 		return dcc;
 		list = list->next;
 	}
 	return 0;
 }
 
-
 static struct DCC *
 find_dcc_from_port (int port, int type)
 {
@@ -1451,7 +1500,7 @@ new_dcc (void)
 }
 
 void
-dcc_chat (struct session *sess, char *nick)
+dcc_chat (struct session *sess, char *nick, int passive)
 {
 	char outbuf[512];
 	struct DCC *dcc;
@@ -1495,7 +1544,7 @@ dcc_chat (struct session *sess, char *nick)
 	dcc->dccstat = STAT_QUEUED;
 	dcc->type = TYPE_CHATSEND;
 	dcc->nick = strdup (nick);
-	if (dcc_listen_init (dcc, sess))
+	if (passive || dcc_listen_init (dcc, sess))
 	{
 		if (prefs.autoopendccchatwindow)
 		{
@@ -1503,8 +1552,17 @@ dcc_chat (struct session *sess, char *nick)
 				fe_dcc_add (dcc);
 		} else
 			fe_dcc_add (dcc);
-		snprintf (outbuf, sizeof (outbuf), "DCC CHAT chat %lu %d",
-						dcc->addr, dcc->port);
+
+		if (passive)
+		{
+			dcc->pasvid = new_id ();
+			snprintf (outbuf, sizeof (outbuf), "DCC CHAT chat 199 %d %d",
+						 dcc->port, dcc->pasvid);
+		} else
+		{
+			snprintf (outbuf, sizeof (outbuf), "DCC CHAT chat %u %d",
+						 dcc->addr, dcc->port);
+		}
 		dcc->serv->p_ctcp (dcc->serv, nick, outbuf);
 		EMIT_SIGNAL (XP_TE_DCCCHATOFFERING, sess, nick, NULL, NULL, NULL, 0);
 	} else
@@ -1563,6 +1621,7 @@ is_resumable (struct DCC *dcc)
 			{
 				if (d != dcc && strcmp (d->destfile, dcc->destfile) == 0)
 				{
+					dcc->resume_error = 3;	/* dccgui.c uses it */
 					dcc->resumable = 0;
 					dcc->pos = 0;
 					break;
@@ -1584,8 +1643,8 @@ dcc_resume (struct DCC *dcc)
 	{
 		/* filename contains spaces? Quote them! */
 		snprintf (tbuf, sizeof (tbuf) - 10, strchr (dcc->file, ' ') ?
-					  "DCC RESUME \"%s\" %d %u" :
-					  "DCC RESUME %s %d %u",
+					  "DCC RESUME \"%s\" %d %"DCC_SFMT :
+					  "DCC RESUME %s %d %"DCC_SFMT,
 					  dcc->file, dcc->port, dcc->resumable);
 
 		if (dcc->pasvid)
@@ -1627,7 +1686,7 @@ dcc_deny_chat (void *ud)
 }
 
 static struct DCC *
-dcc_add_chat (session *sess, char *nick, int port, unsigned long addr, int pasvid)
+dcc_add_chat (session *sess, char *nick, int port, guint32 addr, int pasvid)
 {
 	struct DCC *dcc;
 
@@ -1667,7 +1726,7 @@ dcc_add_chat (session *sess, char *nick, int port, unsigned long addr, int pasvi
 }
 
 static struct DCC *
-dcc_add_file (session *sess, char *file, unsigned long size, int port, char *nick, unsigned long addr, int pasvid)
+dcc_add_file (session *sess, char *file, DCC_SIZE size, int port, char *nick, guint32 addr, int pasvid)
 {
 	struct DCC *dcc;
 	char tbuf[512];
@@ -1737,8 +1796,8 @@ dcc_add_file (session *sess, char *file, unsigned long size, int port, char *nic
 		} else
 			fe_dcc_add (dcc);
 	}
-	sprintf (tbuf, "%lu", size);
-	snprintf (tbuf + 24, 300, "%s:%d", net_ip (dcc->addr), dcc->port);
+	sprintf (tbuf, "%"DCC_SFMT, size);
+	snprintf (tbuf + 24, 300, "%s:%d", net_ip (addr), port);
 	EMIT_SIGNAL (XP_TE_DCCSENDOFFER, sess->server->front_session, nick,
 					 file, tbuf, tbuf + 24, 0);
 
@@ -1753,7 +1812,9 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 	struct DCC *dcc;
 	char *type = word[5];
 	int port, pasvid = 0;
-	unsigned long size, addr;
+	guint32 addr;
+	DCC_SIZE size;
+	int psend = 0;
 
 	if (!strcasecmp (type, "CHAT"))
 	{
@@ -1762,6 +1823,11 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 
 		if (port == 0)
 			pasvid = atoi (word[9]);
+		else if (word[9][0] != 0)
+		{
+			pasvid = atoi (word[9]);
+			psend = 1;
+		}
 
 		if (!addr /*|| (port < 1024 && port != 0)*/
 			|| port > 0xffff || (port == 0 && pasvid == 0))
@@ -1769,6 +1835,22 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 			dcc_malformed (sess, nick, word_eol[4] + 2);
 			return;
 		}
+
+		if (psend)
+		{
+			dcc = find_dcc_from_id (pasvid, TYPE_CHATSEND);
+			if (dcc)
+			{
+				dcc->addr = addr;
+				dcc->port = port;
+				dcc_connect (dcc);
+			} else
+			{
+				dcc_malformed (sess, nick, word_eol[4] + 2);
+			}
+			return;
+		}
+
 		dcc = find_dcc (nick, "", TYPE_CHATSEND);
 		if (dcc)
 			dcc_close (dcc, 0, TRUE);
@@ -1781,7 +1863,7 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 		return;
 	}
 
-	if (!strcasecmp (type, "RESUME"))
+	if (!strcasecmp (type, "Resume"))
 	{
 		port = atoi (word[7]);
 
@@ -1797,7 +1879,7 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 			dcc = find_dcc (nick, word[6], TYPE_SEND);
 		if (dcc)
 		{
-			size = strtoul (word[8], NULL, 10);
+			size = BIG_STR_TO_INT (word[8]);
 			dcc->resumable = size;
 			if (dcc->resumable < dcc->size)
 			{
@@ -1808,24 +1890,24 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 				/* Checking if dcc is passive and if filename contains spaces */
 				if (dcc->pasvid)
 					snprintf (tbuf, sizeof (tbuf), strchr (file_part (dcc->file), ' ') ?
-							"DCC ACCEPT \"%s\" %d %u %d" :
-							"DCC ACCEPT %s %d %u %d",
+							"DCC ACCEPT \"%s\" %d %"DCC_SFMT" %d" :
+							"DCC ACCEPT %s %d %"DCC_SFMT" %d",
 							file_part (dcc->file), port, dcc->resumable, dcc->pasvid);
 				else
 					snprintf (tbuf, sizeof (tbuf), strchr (file_part (dcc->file), ' ') ?
-							"DCC ACCEPT \"%s\" %d %u" :
-							"DCC ACCEPT %s %d %u",
+							"DCC ACCEPT \"%s\" %d %"DCC_SFMT :
+							"DCC ACCEPT %s %d %"DCC_SFMT,
 							file_part (dcc->file), port, dcc->resumable);
 
 				dcc->serv->p_ctcp (dcc->serv, dcc->nick, tbuf);
 			}
-			sprintf (tbuf, "%u", dcc->pos);
+			sprintf (tbuf, "%"DCC_SFMT, dcc->pos);
 			EMIT_SIGNAL (XP_TE_DCCRESUMEREQUEST, sess, nick,
 							 file_part (dcc->file), tbuf, NULL, 0);
 		}
 		return;
 	}
-	if (!strcasecmp (type, "ACCEPT"))
+	if (!strcasecmp (type, "Accept"))
 	{
 		port = atoi (word[7]);
 		dcc = find_dcc_from_port (port, TYPE_RECV);
@@ -1838,11 +1920,10 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 	if (!strcasecmp (type, "SEND"))
 	{
 		char *file = file_part (word[6]);
-		int psend = 0;
 
 		port = atoi (word[8]);
 		addr = strtoul (word[7], NULL, 10);
-		size = strtoul (word[9], NULL, 10);
+		size = BIG_STR_TO_INT (word[9]);
 
 		if (port == 0) /* Passive dcc requested */
 			pasvid = atoi (word[10]);
@@ -1909,7 +1990,7 @@ dcc_show_list (struct session *sess)
 	{
 		dcc = (struct DCC *) list->data;
 		i++;
-		PrintTextf (sess, " %s  %-10.10s %-7.7s %-7u %-7u %s\n",
+		PrintTextf (sess, " %s  %-10.10s %-7.7s %-7"DCC_SFMT" %-7"DCC_SFMT" %s\n",
 					 dcctypes[dcc->type], dcc->nick,
 					 _(dccstat[dcc->dccstat].name), dcc->size, dcc->pos,
 					 file_part (dcc->file));
