@@ -37,6 +37,7 @@
 #include "outbound.h"
 #include "inbound.h"
 #include "server.h"
+#include "servlist.h"
 #include "text.h"
 #include "ctcp.h"
 #include "plugin.h"
@@ -47,7 +48,7 @@
 /* also light/dark gray (14/15) */
 /* 5,7,8 are all shades of yellow which happen to look dman near the same */
 
-static char rcolors[] = { 3, 4, 6, 8, 9, 10, 11, 12, 13 };
+static char rcolors[] = { 19, 20, 22, 24, 25, 26, 27, 28, 29 };
 
 static int
 color_of (char *name)
@@ -63,7 +64,8 @@ color_of (char *name)
 void
 clear_channel (session *sess)
 {
-	strcpy (sess->waitchannel, sess->channel);
+	if (sess->channel[0])
+		strcpy (sess->waitchannel, sess->channel);
 	sess->channel[0] = 0;
 	sess->doing_who = FALSE;
 	sess->done_away_check = FALSE;
@@ -83,7 +85,7 @@ clear_channel (session *sess)
 	}
 
 	fe_clear_channel (sess);
-	clear_user_list (sess);
+	userlist_clear (sess);
 	fe_set_nonchannel (sess, FALSE);
 	fe_set_title (sess);
 }
@@ -109,13 +111,13 @@ find_session_from_nick (char *nick, server *serv)
 
 	if (serv->front_session)
 	{
-		if (find_name (serv->front_session, nick))
+		if (userlist_find (serv->front_session, nick))
 			return serv->front_session;
 	}
 
 	if (current_sess && current_sess->server == serv)
 	{
-		if (find_name (current_sess, nick))
+		if (userlist_find (current_sess, nick))
 			return current_sess;
 	}
 
@@ -124,7 +126,7 @@ find_session_from_nick (char *nick, server *serv)
 		sess = list->data;
 		if (sess->server == serv)
 		{
-			if (find_name (sess, nick))
+			if (userlist_find (sess, nick))
 				return sess;
 		}
 		list = list->next;
@@ -132,15 +134,43 @@ find_session_from_nick (char *nick, server *serv)
 	return 0;
 }
 
-void
-inbound_privmsg (server *serv, char *from, char *ip, char *text)
+static session *
+inbound_open_dialog (server *serv, char *from)
 {
 	session *sess;
 
-	sess = find_dialog (serv, from);
+	sess = new_ircwindow (serv, from, SESS_DIALOG, 0);
+	/* for playing sounds */
+	EMIT_SIGNAL (XP_TE_OPENDIALOG, sess, NULL, NULL, NULL, NULL, 0);
 
-	if (prefs.beepmsg || (sess && sess->beep))
-		fe_beep ();
+	return sess;
+}
+
+static void
+inbound_make_idtext (server *serv, char *idtext, int max, int id)
+{
+	idtext[0] = 0;
+	if (serv->have_idmsg)
+	{
+		if (id)
+		{
+			safe_strcpy (idtext, prefs.irc_id_ytext, max);
+		} else
+		{
+			safe_strcpy (idtext, prefs.irc_id_ntext, max);
+		}
+		/* convert codes like %C,%U to the proper ones */
+		check_special_chars (idtext, TRUE);
+	}
+}
+
+void
+inbound_privmsg (server *serv, char *from, char *ip, char *text, int id)
+{
+	session *sess;
+	char idtext[64];
+
+	sess = find_dialog (serv, from);
 
 	if (sess || prefs.autodialog)
 	{
@@ -148,10 +178,15 @@ inbound_privmsg (server *serv, char *from, char *ip, char *text)
 		if (!sess)
 		{
 			if (flood_check (from, ip, serv, current_sess, 1))
-				sess = new_ircwindow (serv, from, SESS_DIALOG);	/* Create a dialog session */
+				/* Create a dialog session */
+				sess = inbound_open_dialog (serv, from);
 			else
 				sess = serv->server_session;
 		}
+
+		if (prefs.beepmsg || (sess && sess->beep))
+			sound_beep (sess);
+
 		if (ip && ip[0])
 		{
 			if (prefs.logging && sess->logfd != -1 &&
@@ -163,28 +198,37 @@ inbound_privmsg (server *serv, char *from, char *ip, char *text)
 			}
 			set_topic (sess, ip);
 		}
-		inbound_chanmsg (serv, NULL, from, text, FALSE);
+		inbound_chanmsg (serv, NULL, NULL, from, text, FALSE, id);
 		return;
 	}
 
+	inbound_make_idtext (serv, idtext, sizeof (idtext), id);
+
 	sess = find_session_from_nick (from, serv);
 	if (!sess)
 	{
 		sess = serv->front_session;
-		EMIT_SIGNAL (XP_TE_PRIVMSG, sess, from, text, NULL, NULL, 0);
+
+		if (prefs.beepmsg || (sess && sess->beep))
+			sound_beep (sess);
+
+		EMIT_SIGNAL (XP_TE_PRIVMSG, sess, from, text, idtext, NULL, 0);
 		return;
 	}
 
+	if (prefs.beepmsg || sess->beep)
+		sound_beep (sess);
+
 	if (sess->type == SESS_DIALOG)
-		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, NULL, NULL, 0);
+		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, idtext, NULL, 0);
 	else
-		EMIT_SIGNAL (XP_TE_PRIVMSG, sess, from, text, NULL, NULL, 0);
+		EMIT_SIGNAL (XP_TE_PRIVMSG, sess, from, text, idtext, NULL, 0);
 }
 
 static int
 SearchNick (char *text, char *nicks)
 {
-	char S[300];	/* size of bluestring in xchatprefs */
+	char S[300];	/* size of irc_extra_hilight in xchatprefs */
 	char *n;
 	char *p;
 	char *t;
@@ -193,7 +237,7 @@ SearchNick (char *text, char *nicks)
 	if (nicks == NULL)
 		return 0;
 
-	text = strip_color (text);
+	text = strip_color (text, -1, 1, 1);
 
 	safe_strcpy (S, nicks, sizeof (S));
 	n = strtok (S, ",");
@@ -203,7 +247,11 @@ SearchNick (char *text, char *nicks)
 		ns = strlen (n);
 		while ((p = nocasestrstr (t, n)))
 		{
-			if ((p == text || !isalnum (*(p - 1))) && !isalnum (*(p + ns)))
+			char *prev_char = (p == text) ? NULL : g_utf8_prev_char (p);
+			char *next_char = p + ns;
+			if ((!prev_char ||
+			     !g_unichar_isalnum (g_utf8_get_char(prev_char))) &&
+			    !g_unichar_isalnum (g_utf8_get_char(next_char)))
 			{
 				free (text);
 				return 1;
@@ -219,27 +267,54 @@ SearchNick (char *text, char *nicks)
 }
 
 static int
-is_hilight (char *text, session *sess, server *serv)
+FromNick (char *nick, char *nicks)
 {
-	if ((SearchNick (text, serv->nick)) || SearchNick (text, prefs.bluestring))
+	char S[300];	/* size of irc_no_hilight in xchatprefs */
+	char *n;
+	char *t;
+
+	if (nicks == NULL || nicks[0] == 0)
+		return 0;
+
+	safe_strcpy (S, nicks, sizeof (S));
+	n = strtok (S, ",");
+	while (n != NULL)
+	{
+		t = nick;
+		if (nocasestrstr(t, n))
+			return 1;
+		n = strtok (NULL, ",");
+	}
+	return 0;
+}
+
+static int
+is_hilight (char *from, char *text, session *sess, server *serv)
+{
+	if (FromNick(from, prefs.irc_no_hilight))
+		return 0;
+
+	if (SearchNick (text, serv->nick) ||
+		 SearchNick (text, prefs.irc_extra_hilight) ||
+		 FromNick (from, prefs.irc_nick_hilight))
 	{
 		if (sess != current_tab)
-		{
 			sess->nick_said = TRUE;
-			fe_set_hilight (sess);
-		}
+		fe_set_hilight (sess);
 		return 1;
 	}
 	return 0;
 }
 
 void
-inbound_action (session *sess, char *chan, char *from, char *text, int fromme)
+inbound_action (session *sess, char *chan, char *from, char *text, int fromme, int id)
 {
 	session *def = sess;
 	server *serv = sess->server;
 	int beep = FALSE;
+	struct User *user;
 	int hilight = FALSE;
+	char nickchar[2] = "\000";
 
 	if (!fromme)
 	{
@@ -255,7 +330,7 @@ inbound_action (session *sess, char *chan, char *from, char *text, int fromme)
 			sess = find_dialog (serv, from);
 			/* if non found, open a new one */
 			if (!sess && prefs.autodialog)
-				sess = new_ircwindow (serv, from, SESS_DIALOG);
+				sess = inbound_open_dialog (serv, from);
 		}
 	}
 
@@ -275,18 +350,25 @@ inbound_action (session *sess, char *chan, char *from, char *text, int fromme)
 		}
 	}
 
+	user = userlist_find (sess, from);
+	if (user)
+	{
+		nickchar[0] = user->prefix[0];
+		user->lasttalk = time (0);
+	}
+
 	if (!fromme)
 	{
-		hilight = is_hilight (text, sess, serv);
+		hilight = is_hilight (from, text, sess, serv);
 		if (hilight && prefs.beephilight)
 			beep = TRUE;
 
 		if (beep || sess->beep)
-			fe_beep ();
+			sound_beep (sess);
 
 		if (hilight)
 		{
-			EMIT_SIGNAL (XP_TE_HCHANACTION, sess, from, text, NULL, NULL, 0);
+			EMIT_SIGNAL (XP_TE_HCHANACTION, sess, from, text, nickchar, NULL, 0);
 			return;
 		}
 	}
@@ -295,32 +377,35 @@ inbound_action (session *sess, char *chan, char *from, char *text, int fromme)
 	{
 		char tbuf[NICKLEN + 4];
 		snprintf (tbuf, sizeof (tbuf), "\003%d%s", color_of (from), from);
-		EMIT_SIGNAL (XP_TE_CHANACTION, sess, tbuf, text, NULL, NULL, 0);
+		EMIT_SIGNAL (XP_TE_CHANACTION, sess, tbuf, text, nickchar, NULL, 0);
 	} else
 	{
-		EMIT_SIGNAL (XP_TE_CHANACTION, sess, from, text, NULL, NULL, 0);
+		EMIT_SIGNAL (XP_TE_CHANACTION, sess, from, text, nickchar, NULL, 0);
 	}
 }
 
 void
-inbound_chanmsg (server *serv, char *chan, char *from, char *text, char fromme)
+inbound_chanmsg (server *serv, session *sess, char *chan, char *from, char *text, char fromme, int id)
 {
 	struct User *user;
-	session *sess;
 	int hilight = FALSE;
 	char nickchar[2] = "\000";
+	char idtext[64];
 
-	if (chan)
-	{
-		sess = find_channel (serv, chan);
-		if (!sess && !is_channel (serv, chan))
-			sess = find_dialog (serv, chan);
-	} else
-	{
-		sess = find_dialog (serv, from);
-	}
 	if (!sess)
-		return;
+	{
+		if (chan)
+		{
+			sess = find_channel (serv, chan);
+			if (!sess && !is_channel (serv, chan))
+				sess = find_dialog (serv, chan);
+		} else
+		{
+			sess = find_dialog (serv, from);
+		}
+		if (!sess)
+			return;
+	}
 
 	if (sess != current_tab)
 	{
@@ -328,7 +413,7 @@ inbound_chanmsg (server *serv, char *chan, char *from, char *text, char fromme)
 		sess->new_data = FALSE;
 	}
 
-	user = find_name (sess, from);
+	user = userlist_find (sess, from);
 	if (user)
 	{
 		nickchar[0] = user->prefix[0];
@@ -343,28 +428,30 @@ inbound_chanmsg (server *serv, char *chan, char *from, char *text, char fromme)
 		return;
 	}
 
+	inbound_make_idtext (serv, idtext, sizeof (idtext), id);
+
 	if (sess->type != SESS_DIALOG)
 		if (prefs.beepchans || sess->beep)
-			fe_beep ();
+			sound_beep (sess);
 
-	if (is_hilight (text, sess, serv))
+	if (is_hilight (from, text, sess, serv))
 	{
 		hilight = TRUE;
 		if (prefs.beephilight)
-			fe_beep ();
+			sound_beep (sess);
 	}
 	if (sess->type == SESS_DIALOG)
-		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, NULL, NULL, 0);
+		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, idtext, NULL, 0);
 	else if (hilight)
-		EMIT_SIGNAL (XP_TE_HCHANMSG, sess, from, text, nickchar, NULL, 0);
+		EMIT_SIGNAL (XP_TE_HCHANMSG, sess, from, text, nickchar, idtext, 0);
 	else if (prefs.colorednicks)
 	{
 		char tbuf[NICKLEN + 4];
 		snprintf (tbuf, sizeof (tbuf), "\003%d%s", color_of (from), from);
-		EMIT_SIGNAL (XP_TE_CHANMSG, sess, tbuf, text, nickchar, NULL, 0);
+		EMIT_SIGNAL (XP_TE_CHANMSG, sess, tbuf, text, nickchar, idtext, 0);
 	}
 	else
-		EMIT_SIGNAL (XP_TE_CHANMSG, sess, from, text, nickchar, NULL, 0);
+		EMIT_SIGNAL (XP_TE_CHANMSG, sess, from, text, nickchar, idtext, 0);
 }
 
 void
@@ -385,7 +472,7 @@ inbound_newnick (server *serv, char *nick, char *newnick, int quiet)
 		sess = list->data;
 		if (sess->server == serv)
 		{
-			if (change_nick (sess, nick, newnick) || (me && sess->type == SESS_SERVER))
+			if (userlist_change (sess, nick, newnick) || (me && sess->type == SESS_SERVER))
 			{
 				if (!quiet)
 				{
@@ -468,7 +555,7 @@ inbound_ujoin (server *serv, char *chan, char *nick, char *ip)
 			sess = find_unused_session (serv);
 			if (!sess)
 				/* last resort, open a new tab/window */
-				sess = new_ircwindow (serv, chan, SESS_CHANNEL);
+				sess = new_ircwindow (serv, chan, SESS_CHANNEL, 1);
 		}
 	}
 
@@ -477,7 +564,7 @@ inbound_ujoin (server *serv, char *chan, char *nick, char *ip)
 	fe_set_channel (sess);
 	fe_set_title (sess);
 	fe_set_nonchannel (sess, TRUE);
-	clear_user_list (sess);
+	userlist_clear (sess);
 
 	if (prefs.logging)
 		log_open (sess);
@@ -552,7 +639,7 @@ inbound_nameslist (server *serv, char *chan, char *names)
 	if (sess->end_of_names)
 	{
 		sess->end_of_names = FALSE;
-		clear_user_list (sess);
+		userlist_clear (sess);
 	}
 
 	while (1)
@@ -562,12 +649,12 @@ inbound_nameslist (server *serv, char *chan, char *names)
 		case 0:
 			name[pos] = 0;
 			if (pos != 0)
-				add_name (sess, name, 0);
+				userlist_add (sess, name, 0);
 			return;
 		case ' ':
 			name[pos] = 0;
 			pos = 0;
-			add_name (sess, name, 0);
+			userlist_add (sess, name, 0);
 			break;
 		default:
 			name[pos] = *names;
@@ -586,7 +673,7 @@ inbound_topic (server *serv, char *chan, char *topic_text)
 
 	if (sess)
 	{
-		new_topic = strip_color (topic_text);
+		new_topic = strip_color (topic_text, -1, 1, 1);
 		set_topic (sess, new_topic);
 		free (new_topic);
 	} else
@@ -604,7 +691,7 @@ inbound_topicnew (server *serv, char *nick, char *chan, char *topic)
 	sess = find_channel (serv, chan);
 	if (sess)
 	{
-		new_topic = strip_color (topic);
+		new_topic = strip_color (topic, -1, 1, 1);
 		set_topic (sess, new_topic);
 		free (new_topic);
 		EMIT_SIGNAL (XP_TE_NEWTOPIC, sess, nick, topic, chan, NULL, 0);
@@ -619,7 +706,7 @@ inbound_join (server *serv, char *chan, char *user, char *ip)
 	{
 		if (!sess->hide_join_part)
 			EMIT_SIGNAL (XP_TE_JOIN, sess, user, chan, ip, NULL, 0);
-		add_name (sess, user, ip);
+		userlist_add (sess, user, ip);
 	}
 }
 
@@ -630,7 +717,7 @@ inbound_kick (server *serv, char *chan, char *user, char *kicker, char *reason)
 	if (sess)
 	{
 		EMIT_SIGNAL (XP_TE_KICK, sess, kicker, user, chan, reason, 0);
-		sub_name (sess, user);
+		userlist_remove (sess, user);
 	}
 }
 
@@ -647,7 +734,7 @@ inbound_part (server *serv, char *chan, char *user, char *ip, char *reason)
 			else
 				EMIT_SIGNAL (XP_TE_PART, sess, user, ip, chan, NULL, 0);
 		}
-		sub_name (sess, user);
+		userlist_remove (sess, user);
 	}
 }
 
@@ -660,40 +747,10 @@ inbound_topictime (server *serv, char *chan, char *nick, time_t stamp)
 	if (!sess)
 		sess = serv->server_session;
 
-	tim[19] = 0;	/* get rid of the \n */
+	tim[24] = 0;	/* get rid of the \n */
 	EMIT_SIGNAL (XP_TE_TOPICDATE, sess, chan, nick, tim, NULL, 0);
 }
 
-void
-set_server_name (struct server *serv, char *name)
-{
-	GSList *list = sess_list;
-	session *sess;
-
-	if (name[0] == 0)
-		name = serv->hostname;
-
-	safe_strcpy (serv->servername, name, sizeof (serv->servername));
-	while (list)
-	{
-		sess = (session *) list->data;
-		if (sess->server == serv)
-			fe_set_title (sess);
-		list = list->next;
-	}
-	if (serv->server_session->type == SESS_SERVER)
-	{
-		if (serv->networkname)
-		{
-			safe_strcpy (serv->server_session->channel, serv->networkname, CHANLEN);
-		} else
-		{
-			safe_strcpy (serv->server_session->channel, name, CHANLEN);
-		}
-		fe_set_channel (serv->server_session);
-	}
-}
-
 void
 inbound_quit (server *serv, char *nick, char *ip, char *reason)
 {
@@ -708,7 +765,7 @@ inbound_quit (server *serv, char *nick, char *ip, char *reason)
 		{
  			if (sess == current_sess)
  				was_on_front_session = TRUE;
-			if (sub_name (sess, nick))
+			if (userlist_remove (sess, nick))
 			{
 				if (!sess->hide_join_part)
 					EMIT_SIGNAL (XP_TE_QUIT, sess, nick, reason, ip, NULL, 0);
@@ -736,7 +793,7 @@ inbound_ping_reply (session *sess, char *timestring, char *from)
 		lag = 1;
 	}
 
-	sscanf (timestring, "%lu", &tim);
+	tim = strtoul (timestring, NULL, 10);
 	nowtim = make_ping_time ();
 	dif = nowtim - tim;
 
@@ -779,7 +836,7 @@ find_session_from_type (int type, server *serv)
 }
 
 void
-inbound_notice (server *serv, char *to, char *nick, char *msg, char *ip)
+inbound_notice (server *serv, char *to, char *nick, char *msg, char *ip, int id)
 {
 	char *po,*ptr=to;
 	session *sess = 0;
@@ -794,6 +851,12 @@ inbound_notice (server *serv, char *to, char *nick, char *msg, char *ip)
 		sess = find_channel (serv, ptr);
 	}
 
+	if (!sess && ptr[0] == '%')
+	{
+		ptr++;
+		sess = find_channel (serv, ptr);
+	}
+
 	if (!sess && ptr[0] == '+')
 	{
 		ptr++;
@@ -815,14 +878,14 @@ inbound_notice (server *serv, char *to, char *nick, char *msg, char *ip)
 				register unsigned int oldh = prefs.hideuserlist;
 				prefs.hideuserlist = 1;
 				if (stype == SESS_NOTICES)
-					sess = new_ircwindow (serv, "(notices)", SESS_NOTICES);
+					sess = new_ircwindow (serv, "(notices)", SESS_NOTICES, 0);
 				else
-					sess = new_ircwindow (serv, "(snotices)", SESS_SNOTICES);
+					sess = new_ircwindow (serv, "(snotices)", SESS_SNOTICES, 0);
 				prefs.hideuserlist = oldh;
 				fe_set_channel (sess);
 				fe_set_title (sess);
 				fe_set_nonchannel (sess, FALSE);
-				clear_user_list (sess);
+				userlist_clear (sess);
 				if (prefs.logging)
 					log_open (sess);
 			}
@@ -831,7 +894,24 @@ inbound_notice (server *serv, char *to, char *nick, char *msg, char *ip)
 				msg += 14;
 		} else
 		{
-			sess = find_session_from_nick (nick, serv);
+											/* paranoia check */
+			if (msg[0] == '[' && (!serv->have_idmsg || id))
+			{
+				/* guess where chanserv meant to post this -sigh- */
+				if (!strcasecmp (nick, "ChanServ") && !find_dialog (serv, nick))
+				{
+					char *dest = strdup (msg + 1);
+					char *end = strchr (dest, ']');
+					if (end)
+					{
+						*end = 0;
+						sess = find_channel (serv, dest);
+					}
+					free (dest);
+				}
+			}
+			if (!sess)
+				sess = find_session_from_nick (nick, serv);
 		}
 		if (!sess)
 		{
@@ -866,7 +946,7 @@ inbound_notice (server *serv, char *to, char *nick, char *msg, char *ip)
 void
 inbound_away (server *serv, char *nick, char *msg)
 {
-	struct away_msg *away = find_away_message (serv, nick);
+	struct away_msg *away = server_away_find_message (serv, nick);
 	session *sess = NULL;
 	GSList *list;
 
@@ -876,13 +956,18 @@ inbound_away (server *serv, char *nick, char *msg)
 			return;
 	} else
 	{
-		save_away_message (serv, nick, msg);
+		server_away_save_message (serv, nick, msg);
 	}
 
-	if (!serv->inside_whois)
-		sess = find_session_from_nick (nick, serv);
-	if (!sess)
-		sess = serv->server_session;
+	if (prefs.irc_whois_front)
+		sess = serv->front_session;
+	else
+	{
+		if (!serv->inside_whois)
+			sess = find_session_from_nick (nick, serv);
+		if (!sess)
+			sess = serv->server_session;
+	}
 
 	EMIT_SIGNAL (XP_TE_WHOIS5, sess, nick, msg, NULL, NULL, 0);
 
@@ -927,12 +1012,17 @@ inbound_nameslist_end (server *serv, char *chan)
 	return FALSE;
 }
 
-static void
+static gboolean
 check_willjoin_channels (server *serv)
 {
 	char *po;
 	session *sess;
 	GSList *list = sess_list;
+	int i = 0;
+
+	/* shouldnt really happen, the io tag is destroyed in server.c */
+	if (!is_server (serv))
+		return FALSE;
 
 	while (list)
 	{
@@ -950,26 +1040,39 @@ check_willjoin_channels (server *serv)
 				po = strchr (sess->waitchannel, ' ');
 				if (po)
 					*po = 0;
+				i++;
 			}
 		}
 		list = list->next;
 	}
+	serv->joindelay_tag = 0;
+	fe_server_event (serv, FE_SE_LOGGEDIN, i);
+	return FALSE;
 }
 
 void
 inbound_next_nick (session *sess, char *nick)
 {
-	sess->server->nickcount++;
+	char *newnick;
+	server *serv = sess->server;
+	ircnet *net;
 
-	switch (sess->server->nickcount)
+	serv->nickcount++;
+
+	switch (serv->nickcount)
 	{
 	case 2:
-		sess->server->p_change_nick (sess->server, prefs.nick2);
-		EMIT_SIGNAL (XP_TE_NICKCLASH, sess, nick, prefs.nick2, NULL, NULL, 0);
+		newnick = prefs.nick2;
+		net = serv->network;
+		/* use network specific "Second choice"? */
+		if (net && !(net->flags & FLAG_USE_GLOBAL) && net->nick2)
+			newnick = net->nick2;
+		serv->p_change_nick (serv, newnick);
+		EMIT_SIGNAL (XP_TE_NICKCLASH, sess, nick, newnick, NULL, NULL, 0);
 		break;
 
 	case 3:
-		sess->server->p_change_nick (sess->server, prefs.nick3);
+		serv->p_change_nick (serv, prefs.nick3);
 		EMIT_SIGNAL (XP_TE_NICKCLASH, sess, nick, prefs.nick3, NULL, NULL, 0);
 		break;
 
@@ -1009,7 +1112,7 @@ set_default_modes (server *serv)
 
 	if (modes[1] != '\0')
 	{
-		serv->p_nick_mode (serv, serv->nick, modes);
+		serv->p_mode (serv, serv->nick, modes);
 	}
 }
 
@@ -1017,7 +1120,7 @@ void
 inbound_login_start (session *sess, char *nick, char *servname)
 {
 	inbound_newnick (sess->server, sess->server->nick, nick, TRUE);
-	set_server_name (sess->server, servname);
+	server_set_name (sess->server, servname);
 	if (sess->type == SESS_SERVER && prefs.logging)
 		log_open (sess);
 	/* reset our away status */
@@ -1129,8 +1232,8 @@ inbound_user_info (session *sess, char *chan, char *user, char *host,
 	return 1;
 }
 
-void
-inbound_banlist (session *sess, time_t stamp, char *chan, char *mask, char *banner)
+int
+inbound_banlist (session *sess, time_t stamp, char *chan, char *mask, char *banner, int is_exemption)
 {
 	char *time_str = ctime (&stamp);
 	server *serv = sess->server;
@@ -1143,9 +1246,26 @@ inbound_banlist (session *sess, time_t stamp, char *chan, char *mask, char *bann
 	if (!sess)
 		sess = serv->front_session;
    if (!fe_is_banwindow (sess))
+	{
+		/* let proto-irc.c do the 'goto def' for exemptions */
+		if (is_exemption)
+			return FALSE;
+
 		EMIT_SIGNAL (XP_TE_BANLIST, sess, chan, mask, banner, time_str, 0);
-	else
-		fe_add_ban_list (sess, mask, banner, time_str);
+		return TRUE;
+	}
+
+	fe_add_ban_list (sess, mask, banner, time_str, is_exemption);
+	return TRUE;
+}
+
+/* execute 1 end-of-motd command */
+
+static int
+inbound_exec_eom_cmd (char *str, void *sess)
+{
+	handle_command (sess, (str[0] == '/') ? str + 1 : str, TRUE);
+	return 1;
 }
 
 void
@@ -1157,13 +1277,30 @@ inbound_login_end (session *sess, char *text)
 	{
 		if (prefs.ip_from_server && serv->use_who)
 		{
-			serv->skip_next_who = TRUE;
-			serv->p_get_ip (serv, serv->nick);	/* sends WHO mynick */
+			serv->skip_next_userhost = TRUE;
+			serv->p_get_ip_uh (serv, serv->nick);	/* sends USERHOST mynick */
 		}
 		set_default_modes (serv);
-		if (serv->eom_cmd)
-			handle_command (sess, serv->eom_cmd, TRUE);
-		check_willjoin_channels (serv);
+
+		if (serv->network)
+		{
+			/* there may be more than 1, separated by \n */
+			if (((ircnet *)serv->network)->command)
+				token_foreach (((ircnet *)serv->network)->command, '\n',
+									inbound_exec_eom_cmd, sess);
+
+			/* send nickserv password */
+			if (((ircnet *)serv->network)->nickserv)
+				serv->p_ns_identify (serv, ((ircnet *)serv->network)->nickserv);
+		}
+
+		/* send JOIN now or wait? */
+		if (serv->network && ((ircnet *)serv->network)->nickserv &&
+			 prefs.irc_join_delay)
+			serv->joindelay_tag = fe_timeout_add (prefs.irc_join_delay * 1000,
+															  check_willjoin_channels, serv);
+		else
+			check_willjoin_channels (serv);
 		if (serv->supports_watch)
 			notify_send_watches (serv);
 		serv->end_of_motd = TRUE;
@@ -1178,3 +1315,15 @@ inbound_login_end (session *sess, char *text)
 	EMIT_SIGNAL (XP_TE_MOTD, serv->server_session, text, NULL,
 					 NULL, NULL, 0);
 }
+
+void
+inbound_identified (server *serv)	/* 'MODE +e MYSELF' on freenode */
+{
+	if (serv->joindelay_tag)
+	{
+		/* stop waiting, just auto JOIN now */
+		fe_timeout_remove (serv->joindelay_tag);
+		serv->joindelay_tag = 0;
+		check_willjoin_channels (serv);
+	}
+}
