@@ -22,6 +22,7 @@
 #include <stdio.h>
 #include <ctype.h>
 #include <limits.h>
+#include <errno.h>
 
 #define WANTSOCKET
 #define WANTARPA
@@ -391,7 +392,7 @@ ban (session * sess, char *tbuf, char *mask, char *bantypestr, int deop)
 	char username[64], fullhost[128], domain[128], *mode, *p2;
 	server *serv = sess->server;
 
-	user = find_name (sess, mask);
+	user = userlist_find (sess, mask);
 	if (user && user->hostname)  /* it's a nickname, let's find a proper ban mask */
 	{
 		if (deop)
@@ -736,11 +737,12 @@ cmd_dcc (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 			}
 			return FALSE;
 		}
-		if (!strcasecmp (type, "CHAT"))
+		if ((!strcasecmp (type, "CHAT")) || (!strcasecmp (type, "PCHAT")))
 		{
 			char *nick = word[3];
+			int passive = (!strcasecmp(type, "PCHAT")) ? 1 : 0;
 			if (*nick)
-				dcc_chat (sess, nick);
+				dcc_chat (sess, nick, passive);
 			return TRUE;
 		}
 		if (!strcasecmp (type, "LIST"))
@@ -969,6 +971,242 @@ cmd_mdeop (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return TRUE;
 }
 
+GSList *menu_list = NULL;
+
+static void
+menu_free (menu_entry *me)
+{
+	free (me->path);
+	if (me->label)
+		free (me->label);
+	if (me->cmd)
+		free (me->cmd);
+	if (me->ucmd)
+		free (me->ucmd);
+	free (me);
+}
+
+/* strings equal? but ignore underscores */
+
+int
+menu_streq (const char *s1, const char *s2, int def)
+{
+	/* for separators */
+	if (s1 == NULL && s2 == NULL)
+		return 0;
+	if (s1 == NULL || s2 == NULL)
+		return 1;
+	while (*s1)
+	{
+		if (*s1 == '_')
+			s1++;
+		if (*s2 == '_')
+			s2++;
+		if (*s1 != *s2)
+			return 1;
+		s1++;
+		s2++;
+	}
+	if (!*s2)
+		return 0;
+	return def;
+}
+
+static menu_entry *
+menu_entry_find (char *path, char *label)
+{
+	GSList *list;
+	menu_entry *me;
+
+	list = menu_list;
+	while (list)
+	{
+		me = list->data;
+		if (!strcmp (path, me->path))
+		{
+			if (me->label && label && !strcmp (label, me->label))
+				return me;
+		}
+		list = list->next;
+	}
+	return NULL;
+}
+
+static void
+menu_del_children (char *path, char *label)
+{
+	GSList *list, *next;
+	menu_entry *me;
+	char buf[512];
+
+	if (!label)
+		label = "";
+	if (path[0])
+		snprintf (buf, sizeof (buf), "%s/%s", path, label);
+	else
+		snprintf (buf, sizeof (buf), "%s", label);
+
+	list = menu_list;
+	while (list)
+	{
+		me = list->data;
+		next = list->next;
+		if (!menu_streq (buf, me->path, 0))
+		{
+			menu_list = g_slist_remove (menu_list, me);
+			menu_free (me);
+		}
+		list = next;
+	}
+}
+
+static int
+menu_del (char *path, char *label)
+{
+	GSList *list;
+	menu_entry *me;
+
+	list = menu_list;
+	while (list)
+	{
+		me = list->data;
+		if (!menu_streq (me->label, label, 1) && !menu_streq (me->path, path, 1))
+		{
+			menu_list = g_slist_remove (menu_list, me);
+			fe_menu_del (me);
+			menu_free (me);
+			/* delete this item's children, if any */
+			menu_del_children (path, label);
+			return 1;
+		}
+		list = list->next;
+	}
+
+	return 0;
+}
+
+static void
+menu_add (char *path, char *label, char *cmd, char *ucmd, int pos, int state, int enable, int mod, int key)
+{
+	menu_entry *me;
+
+	/* already exists? */
+	me = menu_entry_find (path, label);
+	if (me)
+	{
+		/* update only */
+		me->state = state;
+		me->enable = enable;
+		fe_menu_update (me);
+		return;
+	}
+
+	me = malloc (sizeof (menu_entry));
+	me->pos = pos;
+	me->state = state;
+	me->enable = enable;
+	me->modifier = mod;
+	me->key = key;
+	me->path = strdup (path);
+	me->label = NULL;
+	me->cmd = NULL;
+	me->ucmd = NULL;
+
+	if (label)
+		me->label = strdup (label);
+	if (cmd)
+		me->cmd = strdup (cmd);
+	if (ucmd)
+		me->ucmd = strdup (ucmd);
+
+	fe_menu_add (me);
+	menu_list = g_slist_append (menu_list, me);
+}
+
+static int
+cmd_menu (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	int idx = 2;
+	int len;
+	int pos = -1;
+	int state;
+	int toggle = FALSE;
+	int enable = TRUE;
+	int key = 0;
+	int mod = 0;
+	char *label;
+
+	if (!word[2][0] || !word[3][0])
+		return FALSE;
+
+	/* -eX enabled or not? */
+	if (word[idx][0] == '-' && word[idx][1] == 'e')
+	{
+		enable = atoi (word[idx] + 2);
+		idx++;
+	}
+
+	/* -k<mod>,<key> key binding */
+	if (word[idx][0] == '-' && word[idx][1] == 'k')
+	{
+		char *comma = strchr (word[idx], ',');
+		if (!comma)
+			return FALSE;
+		mod = atoi (word[idx] + 2);
+		key = atoi (comma + 1);
+		idx++;
+	}
+
+	/* -pX to specify menu position */
+	if (word[idx][0] == '-' && word[idx][1] == 'p')
+	{
+		pos = atoi (word[2] + 2);
+		idx++;
+	}
+
+	/* -tX to specify toggle item with default state */
+	if (word[idx][0] == '-' && word[idx][1] == 't')
+	{
+		state = atoi (word[idx] + 2);
+		idx++;
+		toggle = TRUE;
+	}
+
+	/* the path */
+	path_part (word[idx+1], tbuf, 512);
+	len = strlen (tbuf);
+	if (len)
+		tbuf[len - 1] = 0;
+
+	/* the name of the item */
+	label = file_part (word[idx + 1]);
+	if (label[0] == '-' && label[1] == 0)
+		label = NULL;	/* separator */
+
+	if (!strcasecmp (word[idx], "ADD"))
+	{
+		if (toggle)
+		{
+			menu_add (tbuf, label, word[idx + 2], word[idx + 3], pos, state, enable, mod, key);
+		} else
+		{
+			if (word[idx + 2][0])
+				menu_add (tbuf, label, word[idx + 2], NULL, pos, 0, enable, mod, key);
+			else
+				menu_add (tbuf, label, NULL, NULL, pos, 0, enable, mod, key);
+		}
+		return TRUE;
+	}
+
+	if (!strcasecmp (word[idx], "DEL"))
+	{
+		menu_del (tbuf, label);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static int
 mkick_cb (struct User *user, multidata *data)
 {
@@ -1037,7 +1275,7 @@ cmd_dns (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	{
 		if (strchr (nick, '.') == NULL)
 		{
-			user = find_name (sess, nick);
+			user = userlist_find (sess, nick);
 			if (user && user->hostname)
 			{
 				do_dns (sess, user->nick, user->hostname);
@@ -1260,7 +1498,7 @@ cont:				esc = FALSE;
 					k = 0;
 				} else
 				{
-					if (isdigit (buf[i]) && k < (sizeof (numb) - 1))
+					if (isdigit ((unsigned char) buf[i]) && k < (sizeof (numb) - 1))
 					{
 						numb[k] = buf[i];
 						k++;
@@ -1311,7 +1549,7 @@ exec_data (GIOChannel *source, GIOCondition condition, struct nbexec *s)
 	}
 	else
 		readpos = buf = malloc(2050);
-	
+
 	rd = read (sok, readpos, 2048);
 	if (rd < 1)
 	{
@@ -1358,7 +1596,7 @@ exec_data (GIOChannel *source, GIOCondition condition, struct nbexec *s)
 		else
 			PrintText (s->sess, buf);
 	}
-	
+
 	free(buf);
 	return TRUE;
 }
@@ -1401,7 +1639,7 @@ cmd_exec (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		{
 			if (access ("/bin/sh", X_OK) != 0)
 			{
-				fe_message (_("I need /bin/sh to run!\n"), FALSE);
+				fe_message (_("I need /bin/sh to run!\n"), FE_MSG_ERROR);
 				return TRUE;
 			}
 		}
@@ -1595,6 +1833,18 @@ cmd_getstr (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return TRUE;
 }
 
+static int
+cmd_ghost (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	if (word[3][0])
+	{
+		sess->server->p_ns_ghost (sess->server, word[2], word[3]);
+		return TRUE;
+	}
+
+	return FALSE; 
+}
+
 static int
 cmd_gui (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -1622,6 +1872,10 @@ cmd_gui (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 			fe_ctrl_gui (sess, 6, 0);
 		else
 			return FALSE;
+	} else if (!strcasecmp (word[2], "MSGBOX"))
+	{
+		fe_message (word[3], FE_MSG_INFO);
+		return TRUE;
 	} else
 	{
 		return FALSE;
@@ -1630,11 +1884,71 @@ cmd_gui (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return TRUE;
 }
 
+typedef struct
+{
+	int longfmt;
+	int i, t;
+	char *buf;
+} help_list;
+
+static void
+show_help_line (session *sess, help_list *hl, char *name, char *usage)
+{
+	int j, len, max;
+	char *p;
+
+	if (hl->longfmt)	/* long format for /HELP -l */
+	{
+		if (!usage || usage[0] == 0)
+			PrintTextf (sess, "   \0034%s\003 :\n", name);
+		else
+			PrintTextf (sess, "   \0034%s\003 : %s\n", name, _(usage));
+		return;
+	}
+
+	/* append the name into buffer, but convert to uppercase */
+	len = strlen (hl->buf);
+	p = name;
+	while (*p)
+	{
+		hl->buf[len] = toupper ((unsigned char) *p);
+		len++;
+		p++;
+	}
+	hl->buf[len] = 0;
+
+	hl->t++;
+	if (hl->t == 5)
+	{
+		hl->t = 0;
+		strcat (hl->buf, "\n");
+		PrintText (sess, hl->buf);
+		hl->buf[0] = ' ';
+		hl->buf[1] = ' ';
+		hl->buf[2] = 0;
+	} else
+	{
+		/* append some spaces after the command name */
+		max = strlen (name);
+		if (max < 10)
+		{
+			max = 10 - max;
+			for (j = 0; j < max; j++)
+			{
+				hl->buf[len] = ' ';
+				len++;
+				hl->buf[len] = 0;
+			}
+		}
+	}
+}
+
 static int
 cmd_help (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	int i = 0, longfmt = 0;
 	char *helpcmd = "";
+	GSList *list;
 
 	if (tbuf)
 		helpcmd = word[2];
@@ -1647,79 +1961,70 @@ cmd_help (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	} else
 	{
 		struct popup *pop;
-		GSList *list = command_list;
 		char *buf = malloc (4096);
-		int t = 1, j;
-		strcpy (buf, _("\nCommands Available:\n\n  "));
-		if (longfmt)
+		help_list hl;
+
+		hl.longfmt = longfmt;
+		hl.buf = buf;
+
+		PrintTextf (sess, "\n%s\n\n", _("Commands Available:"));
+		buf[0] = ' ';
+		buf[1] = ' ';
+		buf[2] = 0;
+		hl.t = 0;
+		hl.i = 0;
+		while (xc_cmds[i].name)
 		{
-			while (1)
-			{
-				if (!xc_cmds[i].name)
-					break;
-				if (!xc_cmds[i].help || *xc_cmds[i].help == '\0')
-					snprintf (buf, 4096, "   \0034%s\003 :\n", xc_cmds[i].name);
-				else
-					snprintf (buf, 4096, "   \0034%s\003 : %s\n", xc_cmds[i].name,
-								 _(xc_cmds[i].help));
-				PrintText (sess, buf);
-				i++;
-			}
-			buf[0] = 0;
-		} else
-		{
-			while (1)
-			{
-				if (!xc_cmds[i].name)
-					break;
-				strcat (buf, xc_cmds[i].name);
-				t++;
-				if (t == 6)
-				{
-					t = 1;
-					strcat (buf, "\n  ");
-				} else
-					for (j = 0; j < (10 - strlen (xc_cmds[i].name)); j++)
-						strcat (buf, " ");
-				i++;
-			}
+			show_help_line (sess, &hl, xc_cmds[i].name, xc_cmds[i].help);
+			i++;
 		}
-		strcat (buf,
-				  _("\n\nType /HELP <command> for more information, or /HELP -l\n\n"));
-		strcat (buf, _("User defined commands:\n\n  "));
-		t = 1;
+		strcat (buf, "\n");
+		PrintText (sess, buf);
+
+		PrintTextf (sess, "\n%s\n\n", _("User defined commands:"));
+		buf[0] = ' ';
+		buf[1] = ' ';
+		buf[2] = 0;
+		hl.t = 0;
+		hl.i = 0;
+		list = command_list;
 		while (list)
 		{
-			pop = (struct popup *) list->data;
-			strcat (buf, pop->name);
-			t++;
-			if (t == 6)
-			{
-				t = 1;
-				strcat (buf, "\n");
-				PrintText (sess, buf);
-				strcpy (buf, "  ");
-			} else
-			{
-				if (strlen (pop->name) < 10)
-				{
-					for (j = 0; j < (10 - strlen (pop->name)); j++)
-						strcat (buf, " ");
-				}
-			}
+			pop = list->data;
+			show_help_line (sess, &hl, pop->name, pop->cmd);
 			list = list->next;
 		}
 		strcat (buf, "\n");
 		PrintText (sess, buf);
+
+		PrintTextf (sess, "\n%s\n\n", _("Plugin defined commands:"));
+		buf[0] = ' ';
+		buf[1] = ' ';
+		buf[2] = 0;
+		hl.t = 0;
+		hl.i = 0;
+		plugin_command_foreach (sess, &hl, (void *)show_help_line);
+		strcat (buf, "\n");
+		PrintText (sess, buf);
 		free (buf);
 
-		PrintText (sess, "\nPlugin defined commands:\n\n");
-		plugin_show_help (sess, NULL);
-
+		PrintTextf (sess, "\n%s\n\n", _("Type /HELP <command> for more information, or /HELP -l"));
 	}
 	return TRUE;
 }
 
+static int
+cmd_id (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	if (word[2][0])
+	{
+		sess->server->p_ns_identify (sess->server, word[2]);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static int
 cmd_ignore (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -1841,9 +2146,9 @@ cmd_kickban (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	{
 		/* if the reason is a 1 digit number, treat it as a bantype */
 
-		user = find_name (sess, nick);
+		user = userlist_find (sess, nick);
 
-		if (isdigit (reason[0]) && reason[1] == 0)
+		if (isdigit ((unsigned char) reason[0]) && reason[1] == 0)
 		{
 			ban (sess, tbuf, nick, reason, (user && user->op));
 			reason[0] = 0;
@@ -1916,23 +2221,34 @@ cmd_load (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	char *error, *arg, *nl, *file;
 	int len;
 
+	if (!word[2][0])
+		return FALSE;
+
 	if (strcmp (word[2], "-e") == 0)
 	{
 		file = expand_homedir (word[3]);
 		fp = fopen (file, "r");
-		free (file);
-		if (fp)
+		if (!fp)
 		{
-			tbuf[1024] = 0;
-			while (fgets (tbuf, 1024, fp))
-			{
-				nl = strchr (tbuf, '\n');
-				if (nl)
-					*nl = 0;
-				handle_command (sess, tbuf, TRUE);
-			}
-			fclose (fp);
+			PrintTextf (sess, "Cannot access %s\n", file);
+			PrintText (sess, errorstring (errno));
+			free (file);
+			return TRUE;
 		}
+		free (file);
+
+		tbuf[1024] = 0;
+		while (fgets (tbuf, 1024, fp))
+		{
+			nl = strchr (tbuf, '\n');
+			if (nl)
+				*nl = 0;
+			if (tbuf[0] == prefs.cmdchar[0])
+				handle_command (sess, tbuf + 1, TRUE);
+			else
+				handle_command (sess, tbuf, TRUE);
+		}
+		fclose (fp);
 		return TRUE;
 	}
 
@@ -1988,7 +2304,7 @@ cmd_me (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	if (dcc_write_chat (sess->channel, tbuf))
 	{
 		/* print it to screen */
-		inbound_action (sess, sess->channel, sess->server->nick, act, TRUE);
+		inbound_action (sess, sess->channel, sess->server->nick, act, TRUE, FALSE);
 	} else
 	{
 		/* DCC CHAT failed, try through server */
@@ -1996,7 +2312,7 @@ cmd_me (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		{
 			sess->server->p_action (sess->server, sess->channel, act);
 			/* print it to screen */
-			inbound_action (sess, sess->channel, sess->server->nick, act, TRUE);
+			inbound_action (sess, sess->channel, sess->server->nick, act, TRUE, FALSE);
 		} else
 		{
 			notc_msg (sess);
@@ -2011,13 +2327,13 @@ cmd_mode (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	/* +channel channels are dying, let those servers whine about modes.
 	 * return info about current channel if available and no info is given */
-	if ((*word[2] == '+') || (*word[2] == 0) || (!is_channel(sess->server, word[2]) && 
+	if ((*word[2] == '+') || (*word[2] == 0) || (!is_channel(sess->server, word[2]) &&
 				!(rfc_casecmp(sess->server->nick, word[2]) == 0)))
 	{
 		if(sess->channel[0] == 0)
 			return FALSE;
 		sess->server->p_mode (sess->server, sess->channel, word_eol[2]);
-	}	
+	}
 	else
 		sess->server->p_mode (sess->server, word[2], word_eol[3]);
 	return TRUE;
@@ -2133,6 +2449,12 @@ static int
 cmd_newserver (struct session *sess, char *tbuf, char *word[],
 					char *word_eol[])
 {
+	if (strcmp (word[2], "-noconnect") == 0)
+	{
+		new_ircwindow (NULL, word[3], SESS_SERVER, 0);
+		return TRUE;
+	}
+	
 	sess = new_ircwindow (NULL, NULL, SESS_SERVER, 0);
 	cmd_server (sess, tbuf, word, word_eol);
 	return TRUE;
@@ -2280,8 +2602,8 @@ cmd_reconnect (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 				serv->auto_reconnect (serv, TRUE, -1);
 			list = list->next;
 		}
-	}	
-	/* If it isn't "ALL" and there is something 
+	}
+	/* If it isn't "ALL" and there is something
 	there it *should* be a server they are trying to connect to*/
 	else if (*word[2])
 	{
@@ -2308,7 +2630,7 @@ cmd_reconnect (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	else
 	{
 		serv->auto_reconnect (serv, TRUE, -1);
-	}	
+	}
 	prefs.recon_delay = tmp;
 
 	return TRUE;
@@ -2338,6 +2660,39 @@ cmd_say (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	return FALSE;
 }
 
+static int
+cmd_send (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	guint32 addr;
+	socklen_t len;
+	struct sockaddr_in SAddr;
+
+	if (!word[2][0])
+		return FALSE;
+
+	addr = dcc_get_my_address ();
+	if (addr == 0)
+	{
+		/* use the one from our connected server socket */
+		memset (&SAddr, 0, sizeof (struct sockaddr_in));
+		len = sizeof (SAddr);
+		getsockname (sess->server->sok, (struct sockaddr *) &SAddr, &len);
+		addr = SAddr.sin_addr.s_addr;
+	}
+	addr = ntohl (addr);
+
+	if ((addr & 0xffff0000) == 0xc0a80000 ||	/* 192.168.x.x */
+		 (addr & 0xff000000) == 0x0a000000)		/* 10.x.x.x */
+		/* we got a private net address, let's PSEND or it'll fail */
+		snprintf (tbuf, 512, "DCC PSEND %s", word_eol[2]);
+	else
+		snprintf (tbuf, 512, "DCC SEND %s", word_eol[2]);
+
+	handle_command (sess, tbuf, FALSE);
+
+	return TRUE;
+}
+
 static int
 cmd_setcursor (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
@@ -2597,6 +2952,28 @@ userlist_cb (struct User *user, session *sess)
 	return TRUE;
 }
 
+static int
+cmd_uselect (struct session *sess, char *tbuf, char *word[], char *word_eol[])
+{
+	int idx = 2;
+	int clear = TRUE;
+	int scroll = FALSE;
+
+	if (strcmp (word[2], "-a") == 0)	/* ADD (don't clear selections) */
+	{
+		clear = FALSE;
+		idx++;
+	}
+	if (strcmp (word[idx], "-s") == 0)	/* SCROLL TO */
+	{
+		scroll = TRUE;
+		idx++;
+	}
+	/* always valid, no args means clear the selection list */
+	fe_uselect (sess, word + idx, clear, scroll);
+	return TRUE;
+}
+
 static int
 cmd_userlist (struct session *sess, char *tbuf, char *word[],
 				  char *word_eol[])
@@ -2740,10 +3117,12 @@ const struct commands xc_cmds[] = {
 	 N_("CYCLE, parts current channel and immediately rejoins")},
 	{"DCC", cmd_dcc, 0, 0, 1,
 	 N_("\n"
-	 "DCC GET <nick>                     - accept an offered file\n"
-	 "DCC SEND [-maxcps=#] <nick> [file] - send a file to someone\n"
-	 "DCC LIST                           - show DCC list\n"
-	 "DCC CHAT <nick>                    - offer DCC CHAT to someone\n"
+	 "DCC GET <nick>                      - accept an offered file\n"
+	 "DCC SEND [-maxcps=#] <nick> [file]  - send a file to someone\n"
+	 "DCC PSEND [-maxcps=#] <nick> [file] - send a file using passive mode\n"
+	 "DCC LIST                            - show DCC list\n"
+	 "DCC CHAT <nick>                     - offer DCC CHAT to someone\n"
+	 "DCC PCHAT <nick>                    - offer DCC CHAT using passive mode\n"
 	 "DCC CLOSE <type> <nick> <file>         example:\n"
 	 "         /dcc close send johnsmith file.tar.gz")},
 	{"DEBUG", cmd_debug, 0, 0, 1, 0},
@@ -2778,10 +3157,12 @@ const struct commands xc_cmds[] = {
 	 N_("GATE <host> [<port>], proxies through a host, port defaults to 23")},
 	{"GETINT", cmd_getint, 0, 0, 1, "GETINT <default> <command> <prompt>"},
 	{"GETSTR", cmd_getstr, 0, 0, 1, "GETSTR <default> <command> <prompt>"},
-	{"GUI", cmd_gui, 0, 0, 1, "GUI [SHOW|HIDE|FOCUS|FLASH|ICONIFY|MENU TOGGLE|COLOR <n>]"},
+	{"GHOST", cmd_ghost, 1, 0, 1, N_("GHOST <nick> <password>, Kills a ghosted nickname")},
+	{"GUI", cmd_gui, 0, 0, 1, "GUI [SHOW|HIDE|FOCUS|FLASH|ICONIFY|MENU TOGGLE|COLOR <n>|MSGBOX <text>]"},
 	{"HELP", cmd_help, 0, 0, 1, 0},
 	{"HOP", cmd_hop, 1, 1, 1,
 	 N_("HOP <nick>, gives chanhalf-op status to the nick (needs chanop)")},
+	{"ID", cmd_id, 1, 0, 1, N_("ID <password>, identifies yourself to nickserv")},
 	{"IGNORE", cmd_ignore, 0, 0, 1,
 	 N_("IGNORE <mask> <types..> <options..>\n"
 	 "    mask - host mask to ignore, eg: *!*@*.aol.com\n"
@@ -2802,7 +3183,7 @@ const struct commands xc_cmds[] = {
 	{"LASTLOG", cmd_lastlog, 0, 0, 1,
 	 N_("LASTLOG <string>, searches for a string in the buffer")},
 	{"LIST", cmd_list, 1, 0, 1, 0},
-	{"LOAD", cmd_load, 0, 0, 1, N_("LOAD <file>, loads a plugin or script")},
+	{"LOAD", cmd_load, 0, 0, 1, N_("LOAD [-e] <file>, loads a plugin or script")},
 
 	{"MDEHOP", cmd_mdehop, 1, 1, 1,
 	 N_("MDEHOP, Mass deop's all chanhalf-ops in the current channel (needs chanop)")},
@@ -2810,6 +3191,7 @@ const struct commands xc_cmds[] = {
 	 N_("MDEOP, Mass deop's all chanops in the current channel (needs chanop)")},
 	{"ME", cmd_me, 0, 0, 1,
 	 N_("ME <action>, sends the action to the current channel (actions are written in the 3rd person, like /me jumps)")},
+	{"MENU", cmd_menu, 0, 0, 1, "MENU [-eX] [-k<mod>,<key>] [-pX] [-tX] {ADD|DEL} <path> [command] [unselect command]"},
 	{"MKICK", cmd_mkick, 1, 1, 1,
 	 N_("MKICK, Mass kicks everyone except you in the current channel (needs chanop)")},
 	{"MODE", cmd_mode, 1, 0, 1, 0},
@@ -2842,8 +3224,8 @@ const struct commands xc_cmds[] = {
 	 N_("QUOTE <text>, sends the text in raw form to the server")},
 #ifdef USE_OPENSSL
 	{"RECONNECT", cmd_reconnect, 0, 0, 1,
-	 N_("RECONNECT [-ssl] [<host>] [<port>] [<password>], Can be called just as /RECONNECT to reconnect to the current server or with /RECONNECT ALL to reconnect to all the open servers")}, 
-#else 
+	 N_("RECONNECT [-ssl] [<host>] [<port>] [<password>], Can be called just as /RECONNECT to reconnect to the current server or with /RECONNECT ALL to reconnect to all the open servers")},
+#else
 	{"RECONNECT", cmd_reconnect, 0, 0, 1,
 	 N_("RECONNECT [<host>] [<port>] [<password>], Can be called just as /RECONNECT to reconnect to the current server or with /RECONNECT ALL to reconnect to all the open servers")},
 #endif
@@ -2851,6 +3233,7 @@ const struct commands xc_cmds[] = {
 
 	{"SAY", cmd_say, 0, 0, 1,
 	 N_("SAY <text>, sends the text to the object in the current window")},
+	{"SEND", cmd_send, 0, 0, 1, N_("SEND <nick> [<file>]")},
 #ifdef USE_OPENSSL
 	{"SERVCHAN", cmd_servchan, 0, 0, 1,
 	 N_("SERVCHAN [-ssl] <host> <port> <channel>, connects and joins a channel")},
@@ -2876,6 +3259,8 @@ const struct commands xc_cmds[] = {
 	{"UNIGNORE", cmd_unignore, 0, 0, 1, N_("UNIGNORE <mask> [QUIET]")},
 	{"UNLOAD", cmd_unload, 0, 0, 1, N_("UNLOAD <name>, unloads a plugin or script")},
 	{"URL", cmd_url, 0, 0, 1, N_("URL <url>, opens a URL in your browser")},
+	{"USELECT", cmd_uselect, 0, 1, 0,
+	 N_("USELECT [-a] [-s] <nick1> <nick2> etc, highlights nick(s) in channel userlist")},
 	{"USERLIST", cmd_userlist, 1, 1, 1, 0},
 	{"VOICE", cmd_voice, 1, 1, 1,
 	 N_("VOICE <nick>, gives voice status to someone (needs chanop)")},
@@ -2955,9 +3340,9 @@ auto_insert (char *dest, int destlen, unsigned char *src, char *word[],
 	{
 		if (src[0] == '%' || src[0] == '&')
 		{
-			if (isdigit (src[1]))
+			if (isdigit ((unsigned char) src[1]))
 			{
-				if (isdigit (src[2]) && isdigit (src[3]))
+				if (isdigit ((unsigned char) src[2]) && isdigit ((unsigned char) src[3]))
 				{
 					buf[0] = src[1];
 					buf[1] = src[2];
@@ -3110,8 +3495,8 @@ check_special_chars (char *cmd, int do_ascii) /* check for %X */
 				occur++;
 				if (	do_ascii &&
 						j + 3 < len &&
-						(isdigit (cmd[j + 1]) && isdigit (cmd[j + 2]) &&
-						isdigit (cmd[j + 3])))
+						(isdigit ((unsigned char) cmd[j + 1]) && isdigit ((unsigned char) cmd[j + 2]) &&
+						isdigit ((unsigned char) cmd[j + 3])))
 				{
 					tbuf[0] = cmd[j + 1];
 					tbuf[1] = cmd[j + 2];
@@ -3414,7 +3799,7 @@ handle_command (session *sess, char *cmd, int check_spch)
 
 	if (command_level > 99)
 	{
-		fe_message (_("Too many recursive usercommands, aborting."), FALSE);
+		fe_message (_("Too many recursive usercommands, aborting."), FE_MSG_ERROR);
 		return TRUE;
 	}
 	command_level++;
