@@ -17,10 +17,10 @@ N_("%C22*%O$tCannot join%C26 %B$1 %O(You are banned).")},
 {"Change Nick", pevt_changenick_help, 2, 
 N_("%C22*%O$t$1 is now known as $2")},
 
-{"Channel Action", pevt_chanaction_help, 2, 
+{"Channel Action", pevt_chanaction_help, 3, 
 N_("%C18*$t$1%O $2")},
 
-{"Channel Action Hilight", pevt_chanaction_help, 2, 
+{"Channel Action Hilight", pevt_chanaction_help, 3, 
 N_("%C21*%O$t%C21%B$1%O%C21 $2")},
 
 {"Channel Ban", pevt_chanban_help, 2, 
@@ -60,7 +60,7 @@ N_("%C22*%O$t$1 sets mode $2$3 $4")},
 N_("%C22*%O$t%C22Channel $1 modes: $2")},
 
 {"Channel Msg Hilight", pevt_chanmsg_help, 4, 
-N_("$4%C21%B$1%O$t%C21$2")},
+N_("$4%C21%B$1%O%C21$t$2")},
 
 {"Channel Notice", pevt_channotice_help, 3, 
 N_("%C28-%C29$1/$2%C28-%O$t$3")},
@@ -243,7 +243,7 @@ N_("%C21*%O$t%C21$1 has kicked $2 from $3 ($4%O%C21)")},
 N_("%C22*%O$tYou have been killed by $1 ($2%O%C22)")},
 
 {"Message Send", pevt_ctcpsend_help, 2, 
-N_("%C19>%O$1%C19<%O$t$2%O")},
+N_("%C19>%O$1%C19<%O$t$2")},
 
 {"Motd", pevt_servertext_help, 1, 
 N_("%C16*%O$t$1%O")},
@@ -291,7 +291,7 @@ N_("%C22*%O$tNotify: $1 is online ($2).")},
 N_("%C23*%O$t%C23$1 (%O%C23$2) has left $3")},
 
 {"Part with Reason", pevt_partreason_help, 4, 
-N_("%C23*%O$t%C23$1 (%O%C23$2) has left $3 (%O%C23$4)")},
+N_("%C23*%O$t%C23$1 (%O%C23$2) has left $3 (%O%C23%B%B$4%O%C23)")},
 
 {"Ping Reply", pevt_pingrep_help, 2, 
 N_("%C22*%O$tPing reply from $1: $2 second(s)")},
@@ -309,7 +309,7 @@ N_("%C18%B%B$3$1%O$t$2")},
 N_("%C22*%O$tA process is already running")},
 
 {"Quit", pevt_quit_help, 3, 
-N_("%C23*%O$t%C23$1 has quit (%O%C23$2)")},
+N_("%C23*%O$t%C23$1 has quit (%O%C23%B%B$2%O%C23)")},
 
 {"Raw Modes", pevt_rawmodes_help, 2, 
 N_("%C22*%O$t$1 sets modes%B %C30[%O$2%B%C30]")},
@@ -399,13 +399,13 @@ N_("%C23*$tYou have been kicked from $2 by $3 ($4%O%C23)")},
 N_("%C23*$tYou have left channel $3")},
 
 {"You Part with Reason", pevt_partreason_help, 4, 
-N_("%C23*$tYou have left channel $3 (%O%C23$4%O%C23)")},
+N_("%C23*$tYou have left channel $3 (%O%C23%B%B$4%O%C23)")},
 
 {"Your Invitation", pevt_uinvite_help, 3, 
 N_("%C22*%O$tYou've invited%C26 $1%O to%C26 $2%O (%C26$3%O)")},
 
 {"Your Message", pevt_chanmsg_help, 4, 
-N_("%C31%B%B$4$1%O$t%C30$2")},
+N_("%C31%B%B$4$1%O%C30$t$2")},
 
 {"Your Nick Changing", pevt_uchangenick_help, 2, 
 N_("%C22*%O$tYou are now known as $2")},
