@@ -435,8 +435,8 @@ strip_color (char *text, int len, int do_color, int do_attr)
 
 	while (len > 0)
 	{
-		if ((col && isdigit (*text) && nc < 2) ||
-			 (col && *text == ',' && isdigit (*(text+1)) && nc < 3))
+		if ((col && isdigit ((unsigned char) *text) && nc < 2) ||
+			 (col && *text == ',' && isdigit ((unsigned char) *(text+1)) && nc < 3))
 		{
 			nc++;
 			if (*text == ',')
@@ -465,6 +465,7 @@ strip_color (char *text, int len, int do_color, int do_attr)
 			case '\026':			  /*ATTR_REVERSE: */
 			case '\002':			  /*ATTR_BOLD: */
 			case '\037':			  /*ATTR_UNDERLINE: */
+			case '\035':			  /*ATTR_ITALICS: */
 				if (!do_attr)
 				{
 					new_str[i] = *text;
@@ -1084,7 +1085,7 @@ country (char *hostname)
 	char *p;
 	domain_t *dom;
 
-	if (!hostname || !*hostname || isdigit (hostname[strlen (hostname) - 1]))
+	if (!hostname || !*hostname || isdigit ((unsigned char) hostname[strlen (hostname) - 1]))
 		return _("Unknown");
 	if ((p = strrchr (hostname, '.')))
 		p++;
@@ -1156,7 +1157,7 @@ int my_poptParseArgvString(const char * s, int * argcPtr, char *** argvPtr) {
 		if (*src != quote) *buf++ = '\\';
 	    }
 	    *buf++ = *src;
-	} else if (isspace(*src)) {
+	} else if (isspace((unsigned char) *src)) {
 	    if (*argv[argc]) {
 		buf++, argc++;
 		if (argc == argvAlloced) {
@@ -1559,3 +1560,40 @@ mkdir_utf8 (char *dir)
 
 	return ret;
 }
+
+/* separates a string according to a 'sep' char, then calls the callback
+   function for each token found */
+
+int
+token_foreach (char *str, char sep,
+					int (*callback) (char *str, void *ud), void *ud)
+{
+	char t, *start = str;
+
+	while (1)
+	{
+		if (*str == sep || *str == 0)
+		{
+			t = *str;
+			*str = 0;
+			if (callback (start, ud) < 1)
+			{
+				*str = t;
+				return FALSE;
+			}
+			*str = t;
+
+			if (*str == 0)
+				break;
+			str++;
+			start = str;
+
+		} else
+		{
+			/* chars $00-$7f can never be embedded in utf-8 */
+			str++;
+		}
+	}
+
+	return TRUE;
+}
